/// @file   DpmMySql.h
/// @brief  MySQL DPM Implementation.
/// @author Alejandro Álvarez Ayllón <aalvarez@cern.ch>
#ifndef DPMMYSQL_H
#define	DPMMYSQL_H

#include "NsMySql.h"
#include <dmlite/cpp/poolmanager.h>

namespace dmlite {
  
  const int kPoolTypeMax = 16;
  const int kPoolNameMax = 16;
  const int kPoolMetaMax = 1024;

  class DpmMySqlFactory;

  /// Pool manager implementation.
  class MySqlPoolManager: public PoolManager {
   public:
    MySqlPoolManager(DpmMySqlFactory* factory, const std::string& dpmDb) throw (DmException);
    ~MySqlPoolManager();

    std::string getImplId(void) const throw ();

    void setStackInstance(StackInstance* si) throw (DmException);
    void setSecurityContext(const SecurityContext*) throw (DmException);

    std::vector<Pool> getPools(PoolAvailability availability) throw (DmException);
    Pool getPool(const std::string& poolname) throw (DmException);
    
    void newPool(const Pool& pool) throw (DmException);
    void updatePool(const Pool& pool) throw (DmException);
    void deletePool(const Pool& pool) throw (DmException);

    Location whereToRead (const std::string& path) throw (DmException);
    Location whereToRead (ino_t inode)             throw (DmException);
    Location whereToWrite(const std::string& path) throw (DmException);

    void cancelWrite(const Location& loc) throw (DmException);

   protected:
     Location whereToRead(const std::vector<Replica>& replicas) throw (DmException);

   private:
    /// Plugin stack.
    StackInstance* stack_;

    /// DPM DB.
    std::string dpmDb_;

    /// The corresponding factory.
    DpmMySqlFactory* factory_;

    /// Security context
    const SecurityContext* secCtx_;
  };

};

#endif	// DPMMYSQL_H

